from typing import Any

from botocore.exceptions import ClientError as ClientError
from botocore.utils import get_service_module_name as get_service_module_name

class BaseClientExceptions:
    ClientError: Any = ...
    def __init__(self, code_to_exception: Any) -> None: ...
    def from_code(self, error_code: Any) -> Any: ...
    def __getattr__(self, name: Any) -> None: ...

class ClientExceptionsFactory:
    def __init__(self) -> None: ...
    def create_client_exceptions(self, service_model: Any) -> Any: ...
